#include <stdint.h>
#include <math.h>

#include "channels.h"

static float Ks[3];
static float Bs[3];

void set_expo(const float X)
{
	Ks[0] = 1.0 - 1.5 * X;
	Ks[1] = 1.0;
	Ks[2] = 3.0 - 2.0*Ks[0];

	Bs[0] = 0.0;
	Bs[1] = Ks[0] / 2.0;
	Bs[2] = Bs[1] + 0.25;

};

float expo(float val)
{
	float sign;
	float res;

    if (val <= 0)
	{
        sign = -1.0;
		val = -val;
	}
    else
        sign = 1.0;

    if (val < 0.5)
        res =  Ks[0]*val;
    else if (val < 0.75)
        res = (val - 0.5) + Bs[1];
    else
        res = (val - 0.75)*Ks[2] + Bs[2];

    return res*sign;
};

float normalize(int16_t val, const int16_t min, const int16_t zero, const int16_t max)
{
	if (val == zero)
		return 0.0;

	if (val > zero)
	{
		if (val >= max)
			return 1.0;
		return (float)(val - zero)/(float)(max-zero);
	}
	else
	{
		if (val <= min)
			return -1.0;
		return (float)(val - zero)/(float)(zero-min);
	}
};