#include <stdio.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>

#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>

#include "i2cmaster.h"
#include "nunchuk.h"
#include "channels.h"

#define UBRR_38400 12
// INIT USART (RS-232)

void USART_init( uint16_t ubrr )
{
	// USART initialization
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART Receiver: Off
	// USART Transmitter: On
	// USART0 Mode: Asynchronous
	// USART Baud Rate: 38400
	UCSRA=0x00;
	UCSRB=0x08;
	UCSRC=0x06;
	UBRRL=ubrr & 0xFF;
	UBRRH=ubrr >> 8;
}


// SEND A STRING TO THE RS-232
void USART_tx_string( char *data )
{
   while ((*data != '\0'))
   {
      while (!(UCSRA & (1 <<UDRE)));
      UDR = *data;
      data++;
   }
}


void USART_tx( char c )
{
  while (!(UCSRA & (1 <<UDRE)));
  UDR = c;
}

/*
  .

0.    0.
0.  
0.    +1.2 +/- 50 (   )

1.  
1.   +0.3

-   1.5

2.  
2.    +1.2 +/- 50 (   )

3.  
3.   +0.3

-   3

4.  
4.    +1.2 +/- 50 (   )

5.  
5.    +0.3

-   4.5

6.  
6.    24.7   ( 20  ,    .)

-   - 24.7

7.  
7.    +0.3

-   - 25.0

8. -> 0


*/

#define PPM_ENABLE 1

const uint8_t SIMULATOR = 1;

#define PPM_DDR DDRD
#define PPM_PORT PORTD
#define PPM_PIN PD7

#define TIMER_MULT 10

volatile int8_t p_state = 0;
volatile int8_t channels[4];
volatile uint16_t total_len = 1500;

#define ON PPM_PORT |= _BV(PPM_PIN);
#define OFF PPM_PORT &= ~_BV(PPM_PIN);

#ifdef PPM_ENABLE
ISR(TIMER1_COMPA_vect) {
	switch(p_state) {
		case 0:
			TCNT1 = 0;
			ON;
			OCR1A = TCNT1 + (120 + channels[0])*TIMER_MULT;
			p_state = 1;
			break;
		case 1:
			OFF;
			OCR1A = TCNT1 + 30*TIMER_MULT;
			p_state = 2;
			break;
		case 2:
			ON;
			OCR1A = TCNT1 + (120 + channels[1])*TIMER_MULT;
			p_state = 3;
			break;
		case 3:
			OFF;
			OCR1A = TCNT1 + 30*TIMER_MULT;
			p_state = 4;
			break;
		case 4:
			ON;
			OCR1A = TCNT1 + (120 + channels[2])*TIMER_MULT;
			p_state = 5;
			break;
		case 5:
			OFF;
			OCR1A = TCNT1 + 30*TIMER_MULT;
			if (SIMULATOR)
				p_state = 6;
			else
				p_state = 8;
			break;
		case 6:
			ON;
			OCR1A = TCNT1 + (120 + channels[3])*TIMER_MULT;
			p_state = 7;
			break;
		case 7:
			OFF;
			OCR1A = TCNT1 + 30*TIMER_MULT;
			p_state = 8;
			break;
		case 8:
			ON;
			OCR1A = total_len*TIMER_MULT;
			p_state = 9;
			break;
		case 9:
			OFF;
			OCR1A = TCNT1 + 30*TIMER_MULT;
			p_state = 0;
			break;
		default:
			break;
	}


}

#endif // PPM_ENABLE

/*
ISR(PCINT0_vect)
{
	if (p_state == -1)
	{
		if (PINB & _BV(PB0))
		{
			ON;
		}
		else
		{
			OFF;
		}
	}
}
*/

inline void TIMER1_init()
{
	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 1000.000 kHz
	// Mode: Normal top=FFFFh
	// OC1A output: Discon.
	// OC1B output: Discon.
	// Noise Canceler: Off
	// Input Capture on Raising Edge
	// Timer1 Overflow Interrupt: On
	// Input Capture Interrupt: On
	// Compare A Match Interrupt: On
	// Compare B Match Interrupt: Off

	TCCR1A=0x00;
	TCCR1B= _BV(CS11) | _BV(ICNC1);
	TCNT1H=0x00;
	TCNT1L=0x00;
	//ICR1H=0x00;
	//ICR1L=0x00;
	//OCR1AH=0x00;
	//OCR1AL=0x00;
	//OCR1BH=0x00;
	//OCR1BL=0x00;

	// Timer/Counter 1 Interrupt(s) initialization
	//TIMSK1= _BV(ICIE1) | _BV(TOIE1);
	TIMSK= _BV(OCIE1A);

}

/*
inline void PCINT_init()
{
	PCICR |= _BV(PCIE0);
	PCIFR |= _BV(PCIF0);
	PCMSK0 = _BV(PCINT0);
}
*/

int main(void)
{
	char buff[64];

	unsigned char res;
	uint8_t buffer_data[sizeof(nunchuk_data_t)];
	nunchuk_data_t *nunchuk_data = (nunchuk_data_t *)buffer_data;

	uint8_t buffer_axis[sizeof(nunchuk_data_t)];
	nunchuk_axis_t *nunchuk_axis = (nunchuk_axis_t *)buffer_axis;


    USART_init(UBRR_38400);  // Initialize the USART
    _delay_ms(10);

    USART_tx_string("start!\n\r");    // we're alive!

	i2c_init();
	_delay_ms(10);

#ifdef PPM_ENABLE
	PPM_DDR |= _BV(PPM_PIN);	
	TIMER1_init();
#endif //PPM_ENABLE

	//PCINT_init();

	nunchuk_init();

	USART_tx_string("nunchuk init ok\n\r");

	set_expo(0.15);

    sei(); // enable interrupts

	while (1) {

		nunchuk_read(buffer_data);

		if (!SIMULATOR && nunchuk_data->but_c == 0)
		{
			channels[0] = 0;
			channels[1] = 0;
			channels[2] = 0;
			channels[3] = 0;
			p_state = -1;

//			//memset(buff, 0, sizeof(buff));
//			//sprintf(buff, "passthru - c:%d, z:%d\n\r", nunchuk_axis->but_c, nunchuk_axis->but_z);
//			//USART_tx_string(buff);

			_delay_ms(30);
		}
		else {
			if (p_state == -1)
			{
				TIMER1_init();
				p_state = 0;
			}

#ifdef PPM_ENABLE
			while (p_state != 9);
#endif //PPM_ENABLE

			nunchuk_calc_axis(buffer_data, buffer_axis);

			normalize(nunchuk_axis->y, -95, 0, 129);

			// chan 0, Y->, 5 . 
			channels[0] = expo(normalize(nunchuk_axis->y, -95, 0, 129)) * 50.0;

			// chan 1, pitch->elevator, 5 . 
			channels[1] = expo(normalize(nunchuk_axis->pitch, -45, 0, 45)) * 40.0;

			// chan 2, roll->rudder, 5 . 
			channels[2] = expo(normalize(nunchuk_axis->roll, -45, 0, 45)) * 40.0 ;
			
			// chan 3 x -> ch3, 5 
			channels[3] = expo(normalize(nunchuk_axis->x, -95, 0, 129)) * 48.0;


			memset(buff, 0, sizeof(buff));
			sprintf(buff, "Y %3d -> Ch0 %3d ; PITCH %3d -> Ch1 %3d ; ROLL %3d -> Ch2 %3d\n\r", nunchuk_axis->y, channels[0], nunchuk_axis->pitch, channels[1], nunchuk_axis->roll, channels[2]);
			USART_tx_string(buff);

			//_delay_ms(50);

		}


		/*
        memset(buff, 0, sizeof(buff));
		//sprintf(buff, "struct: %x %x %x %x\n\r", buffer[0], buffer[1], buffer[2], buffer[3]);
        sprintf(buff, "JOY X:%3d Y:%3d, BUTT: Z:%d C:%d, ROLL: %4d PITCH: %4d\n\r", nunchuk_axis->x, nunchuk_axis->y, nunchuk_data->but_z, nunchuk_data->but_c, nunchuk_axis->roll, nunchuk_axis->pitch);
		USART_tx_string(buff);

        memset(buff, 0, sizeof(buff));
        sprintf(buff, "channels 1:%3d 2:%3d, 3:%3d\n\r", channels[0], channels[1], channels[2]);
		USART_tx_string(buff);
		*/



		/*
        pos = 0;
        t_state = 0;
        TIMER1_init();

		while (t_state != 2);

        for (uint8_t i=0; i<pos; i++)
        {
            memset(buff, 0, sizeof(buff));
            sprintf(buff, "%u ", intervals[i]/20);
    		USART_tx_string(buff);

        }
        USART_tx_string("\n\r");
		*/


	}

}
